@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 Also see acknowledgements in Readme.html
 
 You may use this sample code for anything you like, it is not covered by the
@@ -17,21 +17,21 @@ same license as the rest of the engine.
 
 #ifndef OGRE_STATIC_LIB
 
-SamplePlugin* sp;
-Sample* s;
+static SamplePlugin* sp;
+static Sample* s;
 
 extern "C" _OgreSampleExport void dllStartPlugin()
 {
-	s = new Sample_Ocean;
-	sp = OGRE_NEW SamplePlugin(s->getInfo()["Title"] + " Sample");
-	sp->addSample(s);
-	Root::getSingleton().installPlugin(sp);
+    s = new Sample_Ocean;
+    sp = OGRE_NEW SamplePlugin(s->getInfo()["Title"] + " Sample");
+    sp->addSample(s);
+    Root::getSingleton().installPlugin(sp);
 }
 
 extern "C" _OgreSampleExport void dllStopPlugin()
 {
-	Root::getSingleton().uninstallPlugin(sp); 
-	OGRE_DELETE sp;
-	delete s;
+    Root::getSingleton().uninstallPlugin(sp); 
+    OGRE_DELETE sp;
+    delete s;
 }
 #endif
